@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id$
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -13,14 +11,27 @@ Print system memory information.
 import psutil
 from psutil._compat import print_
 
-def to_meg(n):
-    return str(int(n / 1024 / 1024)) + "M"
+def bytes2human(n):
+    # http://code.activestate.com/recipes/578019
+    # >>> bytes2human(10000)
+    # '9.8K'
+    # >>> bytes2human(100001221)
+    # '95.4M'
+    symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
+    prefix = {}
+    for i, s in enumerate(symbols):
+        prefix[s] = 1 << (i+1)*10
+    for s in reversed(symbols):
+        if n >= prefix[s]:
+            value = float(n) / prefix[s]
+            return '%.1f%s' % (value, s)
+    return "%sB" % n
 
 def pprint_ntuple(nt):
     for name in nt._fields:
         value = getattr(nt, name)
         if name != 'percent':
-            value = to_meg(value)
+            value = bytes2human(value)
         print_('%-10s : %7s' % (name.capitalize(), value))
 
 def main():
