@@ -1,13 +1,15 @@
 /*
- * $Id$
- *
- * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
  * Linux-specific functions.
  */
 
+
+#ifndef _GNU_SOURCE
+    #define _GNU_SOURCE 1
+#endif
 #include <Python.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -16,14 +18,22 @@
 #include <sched.h>
 #include <sys/syscall.h>
 #include <sys/sysinfo.h>
-#include <linux/unistd.h>
 
 #include "_psutil_linux.h"
 
 
-#define HAS_IOPRIO defined(__NR_ioprio_get) && defined(__NR_ioprio_set)
+ // Linux >= 2.6.13
+#define HAVE_IOPRIO defined(__NR_ioprio_get) && defined(__NR_ioprio_set)
 
-#if HAS_IOPRIO
+// Linux >= 2.6.36; HAVE_PRLIMIT is a macro defined in setup.py
+#ifdef HAVE_PRLIMIT
+    #define _FILE_OFFSET_BITS 64
+    #include <time.h>
+    #include <sys/resource.h>
+#endif
+
+
+#if HAVE_IOPRIO
 enum {
     IOPRIO_WHO_PROCESS = 1,
 };
@@ -95,6 +105,69 @@ linux_ioprio_set(PyObject* self, PyObject* args)
 #endif
 
 
+#ifdef HAVE_PRLIMIT
+/*
+ * A wrapper around prlimit(2); sets process resource limits.
+ * This can be used for both get and set, in which case extra
+ * 'soft' and 'hard' args must be provided.
+ */
+static PyObject*
+linux_prlimit(PyObject* self, PyObject* args)
+{
+    long pid;
+    int ret, resource;
+    struct rlimit old, new;
+    struct rlimit *newp = NULL;
+    PyObject *soft = NULL;
+    PyObject *hard = NULL;
+
+    if (! PyArg_ParseTuple(args, "li|OO", &pid, &resource, &soft, &hard)) {
+        return NULL;
+    }
+
+    // get
+    if (soft == NULL && hard == NULL) {
+        ret = prlimit(pid, resource, NULL, &old);
+        if (ret == -1)
+            return PyErr_SetFromErrno(PyExc_OSError);
+#if defined(HAVE_LONG_LONG)
+        if (sizeof(old.rlim_cur) > sizeof(long)) {
+            return Py_BuildValue("LL", (PY_LONG_LONG)old.rlim_cur,
+                                       (PY_LONG_LONG)old.rlim_max);
+        }
+#endif
+        return Py_BuildValue("ll", (long)old.rlim_cur,
+                                   (long)old.rlim_max);
+    }
+
+    // set
+    else {
+#if defined(HAVE_LARGEFILE_SUPPORT)
+        new.rlim_cur = PyLong_AsLongLong(soft);
+        if (new.rlim_cur == (rlim_t)-1 && PyErr_Occurred())
+            return NULL;
+        new.rlim_max = PyLong_AsLongLong(hard);
+        if (new.rlim_max == (rlim_t)-1 && PyErr_Occurred())
+            return NULL;
+#else
+        new.rlim_cur = PyLong_AsLong(soft);
+        if (new.rlim_cur == (rlim_t)-1 && PyErr_Occurred())
+            return NULL;
+        new.rlim_max = PyLong_AsLong(hard);
+        if (new.rlim_max == (rlim_t)-1 && PyErr_Occurred())
+            return NULL;
+#endif
+        newp = &new;
+        ret = prlimit(pid, resource, newp, &old);
+        if (ret == -1)
+            return PyErr_SetFromErrno(PyExc_OSError);
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+}
+#endif
+
+
 /*
  * Return disk mounted partitions as a list of tuples including device,
  * mount point and filesystem type
@@ -107,12 +180,15 @@ get_disk_partitions(PyObject* self, PyObject* args)
     PyObject* py_retlist = PyList_New(0);
     PyObject* py_tuple = NULL;
 
+    if (py_retlist == NULL)
+        return NULL;
+
     // MOUNTED constant comes from mntent.h and it's == '/etc/mtab'
     Py_BEGIN_ALLOW_THREADS
     file = setmntent(MOUNTED, "r");
     Py_END_ALLOW_THREADS
     if ((file == 0) || (file == NULL)) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromErrnoWithFilename(PyExc_OSError, MOUNTED);
         goto error;
     }
 
@@ -153,6 +229,8 @@ get_sysinfo(PyObject* self, PyObject* args)
     if (sysinfo(&info) != 0) {
         return PyErr_SetFromErrno(PyExc_OSError);
     }
+
+    // note: BOOT_TIME might also be determined from here
     return Py_BuildValue("(KKKKKK)",
         (unsigned long long)info.totalram  * info.mem_unit,   // total
         (unsigned long long)info.freeram   * info.mem_unit,   // free
@@ -160,7 +238,6 @@ get_sysinfo(PyObject* self, PyObject* args)
         (unsigned long long)info.sharedram * info.mem_unit,   // shared
         (unsigned long long)info.totalswap * info.mem_unit,   // swap tot
         (unsigned long long)info.freeswap  * info.mem_unit);  // swap free
-    // TODO: we can also determine BOOT_TIME here
 }
 
 
@@ -216,6 +293,9 @@ get_system_users(PyObject* self, PyObject* args)
     PyObject *user_proc = NULL;
     struct utmp *ut;
 
+    if (ret_list == NULL)
+        return NULL;
+
     setutent();
     while (NULL != (ut = getutent())) {
         tuple = NULL;
@@ -255,12 +335,23 @@ error:
 static PyMethodDef
 PsutilMethods[] =
 {
-#if HAS_IOPRIO
+      // --- per-process functions
+
+#if HAVE_IOPRIO
      {"ioprio_get", linux_ioprio_get, METH_VARARGS,
         "Get process I/O priority"},
      {"ioprio_set", linux_ioprio_set, METH_VARARGS,
         "Set process I/O priority"},
 #endif
+#ifdef HAVE_PRLIMIT
+     {"prlimit", linux_prlimit, METH_VARARGS,
+        "Get or set process resource limits."},
+#endif
+     {"set_process_cpu_affinity", set_process_cpu_affinity, METH_VARARGS,
+        "Set process CPU affinity; expects a bitmask."},
+
+     // --- system related functions
+
      {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
         "Return disk mounted partitions as a list of tuples including "
         "device, mount point and filesystem type"},
@@ -268,8 +359,6 @@ PsutilMethods[] =
         "A wrapper around sysinfo(), return system memory usage statistics"},
      {"get_process_cpu_affinity", get_process_cpu_affinity, METH_VARARGS,
         "Return process CPU affinity as a Python long (the bitmask)."},
-     {"set_process_cpu_affinity", set_process_cpu_affinity, METH_VARARGS,
-        "Set process CPU affinity; expects a bitmask."},
      {"get_system_users", get_system_users, METH_VARARGS,
         "Return currently connected users as a list of tuples"},
 
@@ -329,6 +418,38 @@ void init_psutil_linux(void)
 #else
     PyObject *module = Py_InitModule("_psutil_linux", PsutilMethods);
 #endif
+
+
+#ifdef HAVE_PRLIMIT
+    PyModule_AddIntConstant(module, "RLIM_INFINITY", RLIM_INFINITY);
+    PyModule_AddIntConstant(module, "RLIMIT_AS", RLIMIT_AS);
+    PyModule_AddIntConstant(module, "RLIMIT_CORE", RLIMIT_CORE);
+    PyModule_AddIntConstant(module, "RLIMIT_CPU", RLIMIT_CPU);
+    PyModule_AddIntConstant(module, "RLIMIT_DATA", RLIMIT_DATA);
+    PyModule_AddIntConstant(module, "RLIMIT_FSIZE", RLIMIT_FSIZE);
+    PyModule_AddIntConstant(module, "RLIMIT_LOCKS", RLIMIT_LOCKS);
+    PyModule_AddIntConstant(module, "RLIMIT_MEMLOCK", RLIMIT_MEMLOCK);
+    PyModule_AddIntConstant(module, "RLIMIT_NOFILE", RLIMIT_NOFILE);
+    PyModule_AddIntConstant(module, "RLIMIT_NPROC", RLIMIT_NPROC);
+    PyModule_AddIntConstant(module, "RLIMIT_RSS", RLIMIT_RSS);
+    PyModule_AddIntConstant(module, "RLIMIT_STACK", RLIMIT_STACK);
+    #ifdef RLIMIT_MSGQUEUE
+        PyModule_AddIntConstant(module, "RLIMIT_MSGQUEUE", RLIMIT_MSGQUEUE);
+    #endif
+    #ifdef RLIMIT_NICE
+        PyModule_AddIntConstant(module, "RLIMIT_NICE", RLIMIT_NICE);
+    #endif
+    #ifdef RLIMIT_RTPRIO
+        PyModule_AddIntConstant(module, "RLIMIT_RTPRIO", RLIMIT_RTPRIO);
+    #endif
+    #ifdef RLIMIT_RTTIME
+        PyModule_AddIntConstant(module, "RLIMIT_RTTIME", RLIMIT_RTTIME);
+    #endif
+    #ifdef RLIMIT_SIGPENDING
+        PyModule_AddIntConstant(module, "RLIMIT_SIGPENDING", RLIMIT_SIGPENDING);
+    #endif
+#endif
+
     if (module == NULL) {
         INITERROR;
     }
