@@ -30,10 +30,8 @@ import errno
 import threading
 import tempfile
 import stat
-import collections
 import datetime
 import shutil
-import socket
 import textwrap
 from socket import AF_INET, SOCK_STREAM, SOCK_DGRAM
 try:
@@ -46,7 +44,7 @@ except ImportError:
     ast = None
 
 import psutil
-from psutil._compat import PY3, callable, long, wraps, u
+from psutil._compat import PY3, callable, long, wraps
 
 
 # ===================================================================
@@ -64,6 +62,7 @@ PYTHON = os.path.realpath(sys.executable)
 DEVNULL = open(os.devnull, 'r+')
 TESTFN = os.path.join(os.getcwd(), "$testfile")
 TESTFN_UNICODE = TESTFN + "ƒőő"
+TESTFILE_PREFIX = 'psutil-test-suite-'
 if not PY3:
     try:
         TESTFN_UNICODE = unicode(TESTFN_UNICODE, sys.getfilesystemencoding())
@@ -118,6 +117,28 @@ def get_test_subprocess(cmd=None, stdout=DEVNULL, stderr=DEVNULL, stdin=DEVNULL,
     _subprocesses_started.add(sproc.pid)
     return sproc
 
+_testfiles = []
+
+def pyrun(src):
+    """Run python code 'src' in a separate interpreter.
+    Return interpreter subprocess.
+    """
+    if PY3:
+        src = bytes(src, 'ascii')
+    # could have used NamedTemporaryFile(delete=False) but it's
+    # >= 2.6 only
+    fd, path = tempfile.mkstemp(prefix=TESTFILE_PREFIX)
+    _testfiles.append(path)
+    f = open(path, 'wb')
+    try:
+        f.write(src)
+        f.flush()
+        subp = get_test_subprocess([PYTHON, f.name], stdout=None, stderr=None)
+        wait_for_pid(subp.pid)
+        return subp
+    finally:
+        f.close()
+
 def warn(msg):
     """Raise a warning msg."""
     warnings.warn(msg, UserWarning)
@@ -156,6 +177,27 @@ def which(program):
                 return exe_file
     return None
 
+if POSIX:
+    def get_kernel_version():
+        """Return a tuple such as (2, 6, 36)."""
+        major, minor, micro = os.uname()[2].split('.')[:3]
+        major = int(major)
+        minor = int(minor)
+        try:
+            micro = int(micro)
+        except ValueError:
+            s = ""
+            for x in micro:
+                if x.isdigit():
+                    s += x
+                else:
+                    break
+            if s:
+                micro = int(s)
+            else:
+                micro = 0
+        return (major, minor, micro)
+
 def wait_for_pid(pid, timeout=1):
     """Wait for pid to show up in the process list then return.
     Used in the test suite to give time the sub process to initialize.
@@ -232,8 +274,8 @@ def check_connection(conn):
             s.bind((conn.laddr[0], 0))
             s.close()
     elif conn.family == AF_UNIX:
-       assert not conn.raddr, repr(conn.raddr)
-       assert conn.status == psutil.CONN_NONE, conn.status
+        assert not conn.raddr, repr(conn.raddr)
+        assert conn.status == psutil.CONN_NONE, conn.status
 
     if getattr(conn, 'fd', -1) != -1:
         assert conn.fd > 0, conn
@@ -1031,7 +1073,7 @@ class TestProcess(unittest.TestCase):
         # test wait() against processes which are not our children
         code = "import sys;"
         code += "from subprocess import Popen, PIPE;"
-        code += "cmd = ['%s', '-c', 'import time; time.sleep(2)'];" %PYTHON
+        code += "cmd = ['%s', '-c', 'import time; time.sleep(2)'];" % PYTHON
         code += "sp = Popen(cmd, stdout=PIPE);"
         code += "sys.stdout.write(str(sp.pid));"
         sproc = get_test_subprocess([PYTHON, "-c", code], stdout=subprocess.PIPE)
@@ -1152,7 +1194,7 @@ class TestProcess(unittest.TestCase):
         assert io2.write_bytes >= io1.write_bytes, (io1, io2)
         # test writes
         io1 = p.get_io_counters()
-        f = tempfile.TemporaryFile()
+        f = tempfile.TemporaryFile(prefix=TESTFILE_PREFIX)
         if PY3:
             f.write(bytes("x" * 1000000, 'ascii'))
         else:
@@ -1213,7 +1255,9 @@ class TestProcess(unittest.TestCase):
             self.assertRaises(ValueError, p.set_ionice, 3)
             self.assertRaises(TypeError, p.set_ionice, 2, 1)
 
-    @unittest.skipUnless(LINUX, "feature not supported on this platform")
+    @unittest.skipUnless(LINUX, "only available on Linux")
+    @unittest.skipUnless(get_kernel_version() >= (2, 6, 36),
+        "only available on Linux >= 2.6.36")
     def test_get_rlimit(self):
         import resource
         p = psutil.Process(os.getpid())
@@ -1229,7 +1273,9 @@ class TestProcess(unittest.TestCase):
                 self.assertGreaterEqual(ret[0], -1)
                 self.assertGreaterEqual(ret[1], -1)
 
-    @unittest.skipUnless(LINUX, "feature not supported on this platform")
+    @unittest.skipUnless(LINUX, "only available on Linux")
+    @unittest.skipUnless(get_kernel_version() >= (2, 6, 36),
+        "only available on Linux >= 2.6.36")
     def test_set_rlimit(self):
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
@@ -1642,34 +1688,38 @@ class TestProcess(unittest.TestCase):
             dupsock.close()
 
     def test_get_connections_all(self):
-        tcp_template = "import socket;" \
-                       "s = socket.socket($family, SOCK_STREAM);" \
-                       "s.bind(('$addr', 0));" \
-                       "s.listen(1);" \
-                       "conn, addr = s.accept();"
+        tcp_template = textwrap.dedent("""
+            import socket
+            s = socket.socket($family, socket.SOCK_STREAM)
+            s.bind(('$addr', 0))
+            s.listen(1)
+            conn, addr = s.accept()
+        """)
 
-        udp_template = "import socket, time;" \
-                       "s = socket.socket($family, SOCK_DGRAM);" \
-                       "s.bind(('$addr', 0));" \
-                       "time.sleep(2);"
+        udp_template = textwrap.dedent("""
+            import socket, time
+            s = socket.socket($family, socket.SOCK_DGRAM)
+            s.bind(('$addr', 0))
+            time.sleep(2)
+        """)
 
         from string import Template
-        tcp4_template = Template(tcp_template).substitute(family=AF_INET,
-                                                          addr="127.0.0.1")
-        udp4_template = Template(udp_template).substitute(family=AF_INET,
-                                                          addr="127.0.0.1")
-        tcp6_template = Template(tcp_template).substitute(family=AF_INET6,
-                                                          addr="::1")
-        udp6_template = Template(udp_template).substitute(family=AF_INET6,
-                                                          addr="::1")
+        tcp4_template = Template(tcp_template).substitute(
+            family=int(AF_INET), addr="127.0.0.1")
+        udp4_template = Template(udp_template).substitute(
+            family=int(AF_INET), addr="127.0.0.1")
+        tcp6_template = Template(tcp_template).substitute(
+            family=int(AF_INET6), addr="::1")
+        udp6_template = Template(udp_template).substitute(
+            family=int(AF_INET6), addr="::1")
 
         # launch various subprocess instantiating a socket of various
         # families and types to enrich psutil results
-        tcp4_proc = get_test_subprocess([PYTHON, "-c", tcp4_template])
-        udp4_proc = get_test_subprocess([PYTHON, "-c", udp4_template])
+        tcp4_proc = pyrun(tcp4_template)
+        udp4_proc = pyrun(udp4_template)
         if supports_ipv6():
-            tcp6_proc = get_test_subprocess([PYTHON, "-c", tcp6_template])
-            udp6_proc = get_test_subprocess([PYTHON, "-c", udp6_template])
+            tcp6_proc = pyrun(tcp6_template)
+            udp6_proc = pyrun(udp6_template)
         else:
             tcp6_proc = None
             udp6_proc = None
@@ -1677,60 +1727,42 @@ class TestProcess(unittest.TestCase):
         # check matches against subprocesses just created
         all_kinds = ("all", "inet", "inet4", "inet6", "tcp", "tcp4", "tcp6",
                      "udp", "udp4", "udp6")
+
+        def check_conn(proc, conn, family, type, laddr, raddr, status, kinds):
+            self.assertEqual(conn.family, family)
+            self.assertEqual(conn.type, type)
+            self.assertIn(conn.laddr[0], laddr)
+            self.assertEqual(conn.raddr, raddr)
+            self.assertEqual(conn.status, status)
+            for kind in all_kinds:
+                cons = proc.get_connections(kind=kind)
+                if kind in kinds:
+                    assert cons != [], cons
+                else:
+                    self.assertEqual(cons, [], cons)
+
         for p in psutil.Process(os.getpid()).get_children():
             for conn in p.get_connections():
                 # TCP v4
                 if p.pid == tcp4_proc.pid:
-                    self.assertEqual(conn.family, AF_INET)
-                    self.assertEqual(conn.type, SOCK_STREAM)
-                    self.assertEqual(conn.laddr[0], "127.0.0.1")
-                    self.assertEqual(conn.raddr, ())
-                    self.assertEqual(conn.status, psutil.CONN_LISTEN)
-                    for kind in all_kinds:
-                        cons = p.get_connections(kind=kind)
-                        if kind in ("all", "inet", "inet4", "tcp", "tcp4"):
-                            assert cons != [], cons
-                        else:
-                            self.assertEqual(cons, [], cons)
+                    check_conn(p, conn, AF_INET, SOCK_STREAM, "127.0.0.1", (),
+                               psutil.CONN_LISTEN,
+                               ("all", "inet", "inet4", "tcp", "tcp4"))
                 # UDP v4
                 elif p.pid == udp4_proc.pid:
-                    self.assertEqual(conn.family, AF_INET)
-                    self.assertEqual(conn.type, SOCK_DGRAM)
-                    self.assertEqual(conn.laddr[0], "127.0.0.1")
-                    self.assertEqual(conn.raddr, ())
-                    self.assertEqual(conn.status, psutil.CONN_NONE)
-                    for kind in all_kinds:
-                        cons = p.get_connections(kind=kind)
-                        if kind in ("all", "inet", "inet4", "udp", "udp4"):
-                            assert cons != [], cons
-                        else:
-                            self.assertEqual(cons, [], cons)
+                    check_conn(p, conn, AF_INET, SOCK_DGRAM, "127.0.0.1", (),
+                               psutil.CONN_NONE,
+                               ("all", "inet", "inet4", "udp", "udp4"))
                 # TCP v6
                 elif p.pid == getattr(tcp6_proc, "pid", None):
-                    self.assertEqual(conn.family, AF_INET6)
-                    self.assertEqual(conn.type, SOCK_STREAM)
-                    self.assertIn(conn.laddr[0], ("::", "::1"))
-                    self.assertEqual(conn.raddr, ())
-                    self.assertEqual(conn.status, psutil.CONN_LISTEN)
-                    for kind in all_kinds:
-                        cons = p.get_connections(kind=kind)
-                        if kind in ("all", "inet", "inet6", "tcp", "tcp6"):
-                            assert cons != [], cons
-                        else:
-                            self.assertEqual(cons, [], cons)
+                    check_conn(p, conn, AF_INET6, SOCK_STREAM, ("::", "::1"),
+                               (), psutil.CONN_LISTEN,
+                               ("all", "inet", "inet6", "tcp", "tcp6"))
                 # UDP v6
                 elif p.pid == getattr(udp6_proc, "pid", None):
-                    self.assertEqual(conn.family, AF_INET6)
-                    self.assertEqual(conn.type, SOCK_DGRAM)
-                    self.assertIn(conn.laddr[0], ("::", "::1"))
-                    self.assertEqual(conn.raddr, ())
-                    self.assertEqual(conn.status, psutil.CONN_NONE)
-                    for kind in all_kinds:
-                        cons = p.get_connections(kind=kind)
-                        if kind in ("all", "inet", "inet6", "udp", "udp6"):
-                            assert cons != [], cons
-                        else:
-                            self.assertEqual(cons, [], cons)
+                    check_conn(p, conn, AF_INET6, SOCK_DGRAM, ("::", "::1"),
+                               (), psutil.CONN_NONE,
+                               ("all", "inet", "inet6", "udp", "udp6"))
 
     @unittest.skipUnless(POSIX, 'posix only')
     def test_get_num_fds(self):
@@ -1921,41 +1953,34 @@ class TestProcess(unittest.TestCase):
             # this is the zombie process
             s = socket.socket(socket.AF_UNIX)
             s.connect('%s')
-            s.sendall(str(os.getpid()))
+            if sys.version_info < (3, ):
+                pid = str(os.getpid())
+            else:
+                pid = bytes(str(os.getpid()), 'ascii')
+            s.sendall(pid)
             s.close()
         """ % TESTFN)
-        f = sock = None
+        pyrun(src)
+        sock = None
         try:
-            f = tempfile.NamedTemporaryFile()
-            f.write(src)
-            f.flush()
             sock = socket.socket(socket.AF_UNIX)
             sock.settimeout(2)
             sock.bind(TESTFN)
             sock.listen(1)
-            subp = get_test_subprocess([PYTHON, f.name], stdout=None,
-                                       stderr=None)
             conn, _ = sock.accept()
             zpid = int(conn.recv(1024))
             zproc = psutil.Process(zpid)
             # Make sure we can re-instantiate the process after its
-            # status changed to zombie.
-            # All our API (pid_exists(), Process(), get_pid_list())
-            # is supposed to support zombie processes and treat them
-            # like any other process in a different status.
-            # Reason is we want to support the use case where one
-            # wants to identify and kill all zombie processes on
-            # the system.
+            # status changed to zombie and at least be able to
+            # query its status.
+            # XXX should we also assume ppid should be querable?
             call_until(lambda: zproc.status, "ret == psutil.STATUS_ZOMBIE")
             self.assertTrue(psutil.pid_exists(zpid))
             zproc = psutil.Process(zpid)
-            zproc.as_dict()
             descendants = [x.pid for x in \
                     psutil.Process(os.getpid()).get_children(recursive=True)]
             self.assertIn(zpid, descendants)
         finally:
-            if f is not None:
-                f.close()
             if sock is not None:
                 sock.close()
             reap_children(search_all=True)
@@ -2100,13 +2125,6 @@ class TestFetchAllProcesses(unittest.TestCase):
                                              '.test_' + name))
                     except (psutil.NoSuchProcess, psutil.AccessDenied):
                         err = sys.exc_info()[1]
-                        if isinstance(err, psutil.NoSuchProcess):
-                            if psutil.pid_exists(p.pid):
-                                # XXX race condition; we probably need
-                                # to try figuring out the process
-                                # identity before failing
-                                self.fail("PID still exists but fun raised " \
-                                          "NoSuchProcess")
                         self.assertEqual(err.pid, p.pid)
                         if err.name:
                             # make sure exception's name attr is set
@@ -2347,6 +2365,7 @@ if hasattr(os, 'getuid') and os.getuid() == 0:
                 setattr(self, attr, types.MethodType(test_, self))
 
         def setUp(self):
+            safe_remove(TESTFN)
             os.setegid(1000)
             os.seteuid(1000)
             TestProcess.setUp(self)
@@ -2364,6 +2383,10 @@ if hasattr(os, 'getuid') and os.getuid() == 0:
             else:
                 self.fail("exception not raised")
 
+        def test_zombie_process(self):
+            # causes problems if test test suite is run as root
+            pass
+
 
 # ===================================================================
 # --- Example script tests
@@ -2453,6 +2476,8 @@ def cleanup():
     DEVNULL.close()
     safe_remove(TESTFN)
     safe_rmdir(TESTFN_UNICODE)
+    for path in _testfiles:
+        safe_remove(path)
 
 atexit.register(cleanup)
 safe_remove(TESTFN)
